@@ -1,6 +1,7 @@
-/* Copyright 2003-2004 Roger Dingledine; Copyright 2004 Nick Mathewson */
+/* Copyright 2003-2004 Roger Dingledine
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: util.h,v 1.130 2005/02/25 20:46:13 nickm Exp $ */
+/* $Id: util.h,v 1.132 2005/04/06 19:07:35 arma Exp $ */
 
 /**
  * \file util.h
@@ -9,7 +10,7 @@
 
 #ifndef __UTIL_H
 #define __UTIL_H
-#define UTIL_H_ID "$Id: util.h,v 1.130 2005/02/25 20:46:13 nickm Exp $"
+#define UTIL_H_ID "$Id: util.h,v 1.132 2005/04/06 19:07:35 arma Exp $"
 
 #include "orconfig.h"
 #include "torint.h"
@@ -131,8 +132,8 @@ int tor_inet_ntoa(struct in_addr *in, char *buf, size_t buf_len);
 int is_plausible_address(const char *name);
 
 /* Process helpers */
-void start_daemon(const char *desired_cwd);
-void finish_daemon(void);
+void start_daemon(void);
+void finish_daemon(const char *desired_cwd);
 void write_pidfile(char *filename);
 
 #endif
