@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: buffers.c,v 1.130 2005/03/14 03:18:34 nickm Exp $ */
-const char buffers_c_id[] = "$Id: buffers.c,v 1.130 2005/03/14 03:18:34 nickm Exp $";
+/* $Id: buffers.c,v 1.132 2005/04/07 22:13:16 arma Exp $ */
+const char buffers_c_id[] = "$Id: buffers.c,v 1.132 2005/04/07 22:13:16 arma Exp $";
 
 /**
  * \file buffers.c
@@ -22,8 +22,6 @@ struct buf_t {
 
 /** Size, in bytes, for newly allocated buffers.  Should be a power of 2. */
 #define INITIAL_BUF_SIZE (4*1024)
-/** Maximum size, in bytes, for resized buffers. */
-#define MAX_BUF_SIZE (1024*1024*10)
 /** Size, in bytes, for minimum 'shrink' size for buffers.  Buffers may start
  * out smaller than this, but they will never autoshrink to less
  * than this size. */
