@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitbuild.c,v 1.102 2005/04/01 09:23:51 arma Exp $ */
-const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.102 2005/04/01 09:23:51 arma Exp $";
+/* $Id: circuitbuild.c,v 1.107 2005/04/07 21:07:18 nickm Exp $ */
+const char circuitbuild_c_id[] = "$Id: circuitbuild.c,v 1.107 2005/04/07 21:07:18 nickm Exp $";
 
 /**
  * \file circuitbuild.c
@@ -59,7 +59,7 @@ static uint16_t get_unique_circ_id_by_conn(connection_t *conn) {
       return 0;
     }
     test_circ_id |= high_bit;
-  } while (circuit_get_by_circ_id_conn(test_circ_id, conn));
+  } while (circuit_get_by_circid_orconn(test_circ_id, conn));
   return test_circ_id;
 }
 
@@ -185,7 +185,7 @@ circuit_dump_details(int severity, circuit_t *circ, int poll_index,
                      const char *type, int this_circid, int other_circid) {
   log(severity,"Conn %d has %s circuit: circID %d (other side %d), state %d (%s), born %d:",
       poll_index, type, this_circid, other_circid, circ->state,
-      circuit_state_to_string[circ->state], (int)circ->timestamp_created);
+      circuit_state_to_string(circ->state), (int)circ->timestamp_created);
   if (CIRCUIT_IS_ORIGIN(circ)) { /* circ starts at this node */
     circuit_log_path(severity, circ);
   }
@@ -385,18 +385,20 @@ void circuit_n_conn_done(connection_t *or_conn, int status) {
 static int
 circuit_deliver_create_cell(circuit_t *circ, char *payload) {
   cell_t cell;
+  uint16_t id;
 
   tor_assert(circ);
   tor_assert(circ->n_conn);
   tor_assert(circ->n_conn->type == CONN_TYPE_OR);
   tor_assert(payload);
 
-  circ->n_circ_id = get_unique_circ_id_by_conn(circ->n_conn);
-  if (!circ->n_circ_id) {
+  id = get_unique_circ_id_by_conn(circ->n_conn);
+  if (!id) {
     log_fn(LOG_WARN,"failed to get unique circID.");
     return -1;
   }
-  log_fn(LOG_DEBUG,"Chosen circID %u.",circ->n_circ_id);
+  log_fn(LOG_DEBUG,"Chosen circID %u.", id);
+  circuit_set_circid_orconn(circ, id, circ->n_conn, N_CONN_CHANGED);
 
   memset(&cell, 0, sizeof(cell_t));
   cell.command = CELL_CREATE;
@@ -723,7 +725,7 @@ int circuit_truncated(circuit_t *circ, crypt_path_t *layer) {
         /* no need to send 'end' relay cells,
          * because the other side's already dead
          */
-        connection_close_unattached_ap(stream, END_STREAM_REASON_DESTROY);
+        connection_mark_unattached_ap(stream, END_STREAM_REASON_DESTROY);
       }
     }
 
