@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuituse.c,v 1.64 2005/04/01 08:42:26 arma Exp $ */
-const char circuituse_c_id[] = "$Id: circuituse.c,v 1.64 2005/04/01 08:42:26 arma Exp $";
+/* $Id: circuituse.c,v 1.69 2005/04/07 21:07:19 nickm Exp $ */
+const char circuituse_c_id[] = "$Id: circuituse.c,v 1.69 2005/04/07 21:07:19 nickm Exp $";
 
 /**
  * \file circuituse.c
@@ -240,10 +240,10 @@ void circuit_expire_building(time_t now) {
       if (victim->n_conn)
         log_fn(LOG_INFO,"Abandoning circ %s:%d:%d (state %d:%s, purpose %d)",
                victim->n_conn->address, victim->n_port, victim->n_circ_id,
-               victim->state, circuit_state_to_string[victim->state], victim->purpose);
+               victim->state, circuit_state_to_string(victim->state), victim->purpose);
       else
         log_fn(LOG_INFO,"Abandoning circ %d (state %d:%s, purpose %d)", victim->n_circ_id,
-               victim->state, circuit_state_to_string[victim->state], victim->purpose);
+               victim->state, circuit_state_to_string(victim->state), victim->purpose);
       circuit_log_path(LOG_INFO,victim);
       circuit_mark_for_close(victim);
     }
@@ -417,6 +417,7 @@ void circuit_detach_stream(circuit_t *circ, connection_t *conn) {
   tor_assert(conn);
 
   conn->cpath_layer = NULL; /* make sure we don't keep a stale pointer */
+  conn->on_circuit = NULL;
 
   if (conn == circ->p_streams) {
     circ->p_streams = conn->next_stream;
@@ -483,10 +484,12 @@ void circuit_about_to_close_connection(connection_t *conn) {
       circuit_n_conn_done(conn, 0);
       /* Now close all the attached circuits on it. */
       while ((circ = circuit_get_by_conn(conn))) {
-        if (circ->n_conn == conn) /* it's closing in front of us */
-          circ->n_conn = NULL;
-        if (circ->p_conn == conn) /* it's closing behind us */
-          circ->p_conn = NULL;
+        if (circ->n_conn == conn)
+          /* it's closing in front of us */
+          circuit_set_circid_orconn(circ, 0, NULL, P_CONN_CHANGED);
+        if (circ->p_conn == conn)
+          /* it's closing behind us */
+          circuit_set_circid_orconn(circ, 0, NULL, N_CONN_CHANGED);
         circuit_mark_for_close(circ);
       }
       return;
@@ -498,7 +501,7 @@ void circuit_about_to_close_connection(connection_t *conn) {
        * been sent. But don't kill the circuit.
        */
 
-      circ = circuit_get_by_conn(conn);
+      circ = circuit_get_by_edge_conn(conn);
       if (!circ)
         return;
 
@@ -921,6 +924,7 @@ static void link_apconn_to_circ(connection_t *apconn, circuit_t *circ) {
   log_fn(LOG_DEBUG,"attaching new conn to circ. n_circ_id %d.", circ->n_circ_id);
   apconn->timestamp_lastread = time(NULL); /* reset it, so we can measure circ timeouts */
   apconn->next_stream = circ->p_streams;
+  apconn->on_circuit = circ;
   /* assert_connection_ok(conn, time(NULL)); */
   circ->p_streams = apconn;
 
