@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: config.c,v 1.330 2005/04/01 07:16:52 nickm Exp $ */
-const char config_c_id[] = "$Id: config.c,v 1.330 2005/04/01 07:16:52 nickm Exp $";
+/* $Id: config.c,v 1.333 2005/04/06 19:07:38 arma Exp $ */
+const char config_c_id[] = "$Id: config.c,v 1.333 2005/04/06 19:07:38 arma Exp $";
 
 /**
  * \file config.c
@@ -266,7 +266,7 @@ options_act(void) {
    *    It let us have the parent process stick around until we were sure Tor
    *    was started.  Should we make start_daemon get called earlier? -NM */
   if (options->RunAsDaemon) {
-    start_daemon(options->DataDirectory);
+    start_daemon();
   }
   if (!libevent_initialized) {
     configure_libevent_logging();
@@ -324,7 +324,8 @@ options_act(void) {
   /* Close the temporary log we used while starting up, if it isn't already
    * gone. */
   close_temp_logs();
-  add_callback_log(LOG_NOTICE, LOG_ERR, control_event_logmsg);
+  add_callback_log(LOG_ERR, LOG_ERR, control_event_logmsg);
+  adjust_event_log_severity();
 
   options->_ConnLimit =
     set_max_file_descriptors((unsigned)options->ConnLimit, MAXCONNECTIONS);
@@ -345,7 +346,7 @@ options_act(void) {
   /* Finish backgrounding the process */
   if (options->RunAsDaemon) {
     /* We may be calling this for the n'th time (on SIGHUP), but it's safe. */
-    finish_daemon();
+    finish_daemon(options->DataDirectory);
   }
 
   /* Write our pid to the pid file. If we do not have write permissions we
