@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_or.c,v 1.163 2005/03/31 23:30:13 nickm Exp $ */
-const char connection_or_c_id[] = "$Id: connection_or.c,v 1.163 2005/03/31 23:30:13 nickm Exp $";
+/* $Id: connection_or.c,v 1.168 2005/04/06 21:09:47 nickm Exp $ */
+const char connection_or_c_id[] = "$Id: connection_or.c,v 1.168 2005/04/06 21:09:47 nickm Exp $";
 
 /**
  * \file connection_or.c
@@ -436,6 +436,8 @@ connection_tls_finish_handshake(connection_t *conn) {
   crypto_pk_env_t *identity_rcvd=NULL;
   char digest_rcvd[DIGEST_LEN];
   or_options_t *options = get_options();
+  int severity = (authdir_mode(options) || !server_mode(options))
+                 ? LOG_WARN : LOG_INFO;
 
   conn->state = OR_CONN_STATE_OPEN;
   connection_watch_events(conn, EV_READ);
@@ -479,7 +481,9 @@ connection_tls_finish_handshake(connection_t *conn) {
   if (router && /* we know this nickname */
       router->is_verified && /* make sure it's the right guy */
       memcmp(digest_rcvd, router->identity_digest, DIGEST_LEN) != 0) {
-    log_fn(LOG_WARN, "Identity key not as expected for router claiming to be '%s' (%s:%d) ", nickname, conn->address, conn->port);
+    log_fn(severity,
+           "Identity key not as expected for router claiming to be '%s' (%s:%d)",
+           nickname, conn->address, conn->port);
     return -1;
   }
 #if 0
@@ -494,20 +498,27 @@ connection_tls_finish_handshake(connection_t *conn) {
 #endif
 
   if (connection_or_nonopen_was_started_here(conn)) {
-    /* I initiated this connection. */
+    if (authdir_mode(options)) {
+      /* We initiated this connection to address:port.  Drop all routers
+       * with the same address:port and a different key or nickname.
+       */
+      dirserv_orconn_tls_done(conn->address, conn->port,
+                              digest_rcvd, nickname);
+    }
     if (conn->nickname[0] == '$') {
       /* I was aiming for a particular digest. Did I get it? */
       char d[HEX_DIGEST_LEN+1];
       base16_encode(d, HEX_DIGEST_LEN+1, digest_rcvd, DIGEST_LEN);
       if (strcasecmp(d,conn->nickname+1)) {
-        log_fn(LOG_WARN, "Identity key not as expected for router at %s:%d: wanted %s but got %s",
+        log_fn(severity,
+               "Identity key not as expected for router at %s:%d: wanted %s but got %s",
                conn->address, conn->port, conn->nickname, d);
         control_event_or_conn_status(conn, OR_CONN_EVENT_FAILED);
         return -1;
       }
     } else if (strcasecmp(conn->nickname, nickname)) {
       /* I was aiming for a nickname.  Did I get it? */
-      log_fn(authdir_mode(options) ? LOG_WARN : LOG_INFO,
+      log_fn(severity,
              "Other side (%s:%d) is '%s', but we tried to connect to '%s'",
              conn->address, conn->port, nickname, conn->nickname);
       control_event_or_conn_status(conn, OR_CONN_EVENT_FAILED);
