@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: cpuworker.c,v 1.67 2005/02/22 08:18:35 nickm Exp $ */
-const char cpuworker_c_id[] = "$Id: cpuworker.c,v 1.67 2005/02/22 08:18:35 nickm Exp $";
+/* $Id: cpuworker.c,v 1.75 2005/04/08 04:59:34 nickm Exp $ */
+const char cpuworker_c_id[] = "$Id: cpuworker.c,v 1.75 2005/04/08 04:59:34 nickm Exp $";
 
 /**
  * \file cpuworker.c
@@ -141,12 +141,12 @@ int connection_cpu_process_inbuf(connection_t *conn) {
     /* parse out the circ it was talking about */
     tag_unpack(buf, &addr, &port, &circ_id);
     circ = NULL;
-    /* (Here we use connection_exact_get_by_addr_port rather than
+    /* (Here we use connection_or_exact_get_by_addr_port rather than
      * get_by_identity_digest: we want a specific port here in
      * case there are multiple connections.) */
-    p_conn = connection_exact_get_by_addr_port(addr,port);
+    p_conn = connection_or_exact_get_by_addr_port(addr,port);
     if (p_conn)
-      circ = circuit_get_by_circ_id_conn(circ_id, p_conn);
+      circ = circuit_get_by_circid_orconn(circ_id, p_conn);
 
     if (success == 0) {
       log_fn(LOG_INFO,"decoding onionskin failed. Closing.");
@@ -262,7 +262,7 @@ static int cpuworker_main(void *data) {
       }
       if (write_all(fd, buf, LEN_ONION_RESPONSE, 1) != LEN_ONION_RESPONSE) {
         log_fn(LOG_ERR,"writing response buf failed. Exiting.");
-        spawn_exit();
+        goto end;
       }
       log_fn(LOG_DEBUG,"finished writing response.");
     }
@@ -272,6 +272,7 @@ static int cpuworker_main(void *data) {
     crypto_free_pk_env(onion_key);
   if (last_onion_key)
     crypto_free_pk_env(last_onion_key);
+  tor_close_socket(fd);
   spawn_exit();
   return 0; /* windows wants this function to return an int */
 }
@@ -355,8 +356,36 @@ static void process_pending_task(connection_t *cpuworker) {
     log_fn(LOG_WARN,"assign_to_cpuworker failed. Ignoring.");
 }
 
-/** if cpuworker is defined, assert that he's idle, and use him. else,
- * look for an idle cpuworker and use him. if none idle, queue task onto
+#define CPUWORKER_BUSY_TIMEOUT 100 /* seconds */
+
+/** We have a bug that I can't find. Sometimes, very rarely, cpuworkers
+ * get stuck in the 'busy' state, even though the cpuworker process
+ * thinks of itself as idle. I don't know why. But here's a workaround
+ * to kill any cpuworker that's been busy for more than 100 seconds. */
+static void
+cull_wedged_cpuworkers(void) {
+  connection_t **carray;
+  connection_t *conn;
+  int n_conns, i;
+  time_t now = time(NULL);
+
+  get_connection_array(&carray, &n_conns);
+  for (i = 0; i < n_conns; ++i) {
+    conn = carray[i];
+    if (!conn->marked_for_close &&
+        conn->type == CONN_TYPE_CPUWORKER &&
+        conn->state == CPUWORKER_STATE_BUSY_ONION &&
+        conn->timestamp_lastwritten + CPUWORKER_BUSY_TIMEOUT < now) {
+      log_fn(LOG_NOTICE,"Bug: closing wedged cpuworker. Can somebody find the bug?");
+      num_cpuworkers_busy--;
+      num_cpuworkers--;
+      connection_mark_for_close(conn);
+    }
+  }
+}
+
+/** If cpuworker is defined, assert that he's idle, and use him. Else,
+ * look for an idle cpuworker and use him. If none idle, queue task onto
  * the pending onion list and return.
  * If question_type is CPUWORKER_TASK_ONION then task is a circ.
  * No other question_types are allowed.
@@ -368,6 +397,9 @@ int assign_to_cpuworker(connection_t *cpuworker, unsigned char question_type,
 
   tor_assert(question_type == CPUWORKER_TASK_ONION);
 
+  cull_wedged_cpuworkers();
+  spawn_enough_cpuworkers();
+
   if (question_type == CPUWORKER_TASK_ONION) {
     circ = task;
 
