@@ -1,8 +1,8 @@
 /* Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dirserv.c,v 1.148 2005/03/23 19:15:10 nickm Exp $ */
-const char dirserv_c_id[] = "$Id: dirserv.c,v 1.148 2005/03/23 19:15:10 nickm Exp $";
+/* $Id: dirserv.c,v 1.157 2005/04/07 21:07:19 nickm Exp $ */
+const char dirserv_c_id[] = "$Id: dirserv.c,v 1.157 2005/04/07 21:07:19 nickm Exp $";
 
 #include "or.h"
 
@@ -12,7 +12,7 @@ const char dirserv_c_id[] = "$Id: dirserv.c,v 1.148 2005/03/23 19:15:10 nickm Ex
  **/
 
 /** How far in the future do we allow a router to get? (seconds) */
-#define ROUTER_ALLOW_SKEW (30*60)
+#define ROUTER_ALLOW_SKEW (60*60*12) /* 12 hours */
 /** How many seconds do we wait before regenerating the directory? */
 #define DIR_REGEN_SLACK_TIME 10
 
@@ -343,8 +343,7 @@ dirserv_add_descriptor(const char **desc, const char **msg)
     routerinfo_free(ri);
     *desc = end;
     return -1;
-  }
-  if (r==0) {
+  } else if (r==0) {
     char fp[FINGERPRINT_LEN+1];
     log_fn(LOG_INFO, "Unknown nickname '%s' (%s:%d). Will try to add.",
            ri->nickname, ri->address, ri->or_port);
@@ -358,14 +357,16 @@ dirserv_add_descriptor(const char **desc, const char **msg)
   /* Is there too much clock skew? */
   now = time(NULL);
   if (ri->published_on > now+ROUTER_ALLOW_SKEW) {
-    log_fn(LOG_NOTICE, "Publication time for nickname '%s' is too far in the future; possible clock skew. Not adding.", ri->nickname);
+    log_fn(LOG_NOTICE, "Publication time for nickname '%s' is too far (%d minutes) in the future; possible clock skew. Not adding.",
+           ri->nickname, (int)((ri->published_on-now)/60));
     *msg = "Rejected: Your clock is set too far in the future, or your timezone is not correct.";
     routerinfo_free(ri);
     *desc = end;
     return -1;
   }
   if (ri->published_on < now-ROUTER_MAX_AGE) {
-    log_fn(LOG_NOTICE, "Publication time for router with nickname '%s' is too far in the past. Not adding.", ri->nickname);
+    log_fn(LOG_NOTICE, "Publication time for router with nickname '%s' is too far (%d minutes) in the past. Not adding.",
+           ri->nickname, (int)((now-ri->published_on)/60));
     *msg = "Rejected: Server is expired, or your clock is too far in the past, or your timezone is not correct.";
     routerinfo_free(ri);
     *desc = end;
@@ -382,16 +383,18 @@ dirserv_add_descriptor(const char **desc, const char **msg)
   /* Do we already have an entry for this router? */
   for (i = 0; i < smartlist_len(descriptor_list); ++i) {
     ri_old = smartlist_get(descriptor_list, i);
-    if (!strcasecmp(ri->nickname, ri_old->nickname)) {
+    if (!memcmp(ri->identity_digest, ri_old->identity_digest, DIGEST_LEN)) {
       found = i;
       break;
     }
   }
   if (found >= 0) {
+    char hex_digest[HEX_DIGEST_LEN+1];
+    base16_encode(hex_digest, HEX_DIGEST_LEN+1, ri->identity_digest,DIGEST_LEN);
     /* if so, decide whether to update it. */
     if (ri_old->published_on >= ri->published_on) {
       /* We already have a newer or equal-time descriptor */
-      log_fn(LOG_INFO,"We already have a new enough desc for nickname '%s'. Not adding.",ri->nickname);
+      log_fn(LOG_INFO,"We already have a new enough desc for server %s (nickname '%s'). Not adding.",hex_digest,ri->nickname);
       *msg = "We already have a newer descriptor.";
       /* This isn't really an error; return success. */
       routerinfo_free(ri);
@@ -399,7 +402,7 @@ dirserv_add_descriptor(const char **desc, const char **msg)
       return verified;
     }
     /* We don't alrady have a newer one; we'll update this one. */
-    log_fn(LOG_INFO,"Dirserv updating desc for nickname '%s'",ri->nickname);
+    log_fn(LOG_INFO,"Dirserv updating desc for server %s (nickname '%s')",hex_digest,ri->nickname);
     *msg = verified?"Verified server updated":"Unverified server updated. (Have you sent us your key fingerprint?)";
     routerinfo_free(ri_old);
     smartlist_del_keeporder(descriptor_list, found);
@@ -409,7 +412,8 @@ dirserv_add_descriptor(const char **desc, const char **msg)
     *msg = verified?"Verified server added":"Unverified server added. (Have you sent us your key fingerprint?)";
   }
 
-  ri->is_verified = verified;
+  ri->is_verified = verified ||
+                    tor_version_as_new_as(ri->platform,"0.1.0.2-rc");
   smartlist_add(descriptor_list, ri);
 
   *desc = end;
@@ -426,17 +430,25 @@ static void
 directory_remove_unrecognized(void)
 {
   int i;
+  int r;
   routerinfo_t *ent;
   if (!descriptor_list)
     descriptor_list = smartlist_create();
 
   for (i = 0; i < smartlist_len(descriptor_list); ++i) {
     ent = smartlist_get(descriptor_list, i);
-    if (dirserv_router_fingerprint_is_known(ent)<=0) {
-      log(LOG_INFO, "Router '%s' is no longer recognized",
+    r = dirserv_router_fingerprint_is_known(ent);
+    if (r<0) {
+      log(LOG_INFO, "Router '%s' is now verified with a key; removing old router with same name and different key.",
           ent->nickname);
       routerinfo_free(ent);
       smartlist_del(descriptor_list, i--);
+    } else if (r>0 && !ent->is_verified) {
+      log(LOG_INFO, "Router '%s' is now approved.", ent->nickname);
+      ent->is_verified = 1;
+    } else if (r==0 && ent->is_verified) {
+      log(LOG_INFO, "Router '%s' is no longer approved.", ent->nickname);
+      ent->is_verified = 0;
     }
   }
 }
@@ -983,6 +995,53 @@ size_t dirserv_get_runningrouters(const char **rr, int compress)
   return compress ? the_runningrouters_z_len : the_runningrouters_len;
 }
 
+/** Called when a TLS handshake has completed successfully with a
+ * router listening at <b>address</b>:<b>or_port</b>, and has yielded
+ * a certificate with digest <b>digest_rcvd</b> and nickname
+ * <b>nickname_rcvd</b>.  When this happens, it's clear that any other
+ * descriptors for that address/port combination must be unusable:
+ * delete them if they are not verified.
+ */
+void
+dirserv_orconn_tls_done(const char *address,
+                        uint16_t or_port,
+                        const char *digest_rcvd,
+                        const char *nickname_rcvd)
+{
+  int i;
+  tor_assert(address);
+  tor_assert(digest_rcvd);
+  tor_assert(nickname_rcvd);
+
+  if (!descriptor_list)
+    return;
+
+  for (i = 0; i < smartlist_len(descriptor_list); ++i) {
+    routerinfo_t *ri = smartlist_get(descriptor_list, i);
+    int drop = 0;
+    if (ri->is_verified)
+      continue;
+    if (!strcasecmp(address, ri->address) &&
+        or_port == ri->or_port) {
+      /* We have a router at the same address! */
+      if (strcasecmp(ri->nickname, nickname_rcvd)) {
+        log_fn(LOG_WARN, "Dropping descriptor: nickname '%s' does not match nickname '%s' in cert from %s:%d",
+               ri->nickname, nickname_rcvd, address, or_port);
+        drop = 1;
+      } else if (memcmp(ri->identity_digest, digest_rcvd, DIGEST_LEN)) {
+        log_fn(LOG_WARN, "Dropping descriptor: identity key does not match key in cert from %s:%d",
+               address, or_port);
+        drop = 1;
+      }
+      if (drop) {
+        routerinfo_free(ri);
+        smartlist_del(descriptor_list, i--);
+        directory_set_dirty();
+      }
+    }
+  }
+}
+
 void
 dirserv_free_all(void)
 {
