@@ -1,8 +1,8 @@
 /* Copyright 2003-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: dns.c,v 1.139 2005/03/14 04:42:52 arma Exp $ */
-const char dns_c_id[] = "$Id: dns.c,v 1.139 2005/03/14 04:42:52 arma Exp $";
+/* $Id: dns.c,v 1.147 2005/04/06 20:25:21 arma Exp $ */
+const char dns_c_id[] = "$Id: dns.c,v 1.147 2005/04/06 20:25:21 arma Exp $";
 
 /**
  * \file dns.c
@@ -81,7 +81,7 @@ static SPLAY_HEAD(cache_tree, cached_resolve) cache_root;
 static int compare_cached_resolves(struct cached_resolve *a,
                                    struct cached_resolve *b) {
   /* make this smarter one day? */
-  return strncasecmp(a->address, b->address, MAX_ADDRESSLEN);
+  return strncmp(a->address, b->address, MAX_ADDRESSLEN);
 }
 
 SPLAY_PROTOTYPE(cache_tree, cached_resolve, node, compare_cached_resolves);
@@ -156,7 +156,7 @@ static void purge_expired_resolves(uint32_t now) {
         pendconn = pend->conn;
         connection_edge_end(pendconn, END_STREAM_REASON_TIMEOUT,
                             pendconn->cpath_layer);
-        circuit_detach_stream(circuit_get_by_conn(pendconn), pendconn);
+        circuit_detach_stream(circuit_get_by_edge_conn(pendconn), pendconn);
         connection_free(pendconn);
         tor_free(pend);
       }
@@ -192,7 +192,7 @@ static void send_resolved_cell(connection_t *conn, uint8_t answer_type)
     default:
       tor_assert(0);
     }
-  connection_edge_send_command(conn, circuit_get_by_conn(conn),
+  connection_edge_send_command(conn, circuit_get_by_edge_conn(conn),
                                RELAY_COMMAND_RESOLVED, buf, buflen,
                                conn->cpath_layer);
 }
@@ -246,6 +246,9 @@ int dns_resolve(connection_t *exitconn) {
    * resolves in the tree. */
   purge_expired_resolves(now);
 
+  /* lower-case exitconn->address, so it's in canonical form */
+  tor_strlower(exitconn->address);
+
   /* now check the tree to see if 'address' is already there. */
   strlcpy(search.address, exitconn->address, sizeof(search.address));
   resolve = SPLAY_FIND(cache_tree, &cache_root, &search);
@@ -274,7 +277,7 @@ int dns_resolve(connection_t *exitconn) {
                exitconn->s, exitconn->address);
         if (exitconn->purpose == EXIT_PURPOSE_RESOLVE)
           send_resolved_cell(exitconn, RESOLVED_TYPE_ERROR);
-        circ = circuit_get_by_conn(exitconn);
+        circ = circuit_get_by_edge_conn(exitconn);
         if (circ)
           circuit_detach_stream(circ, exitconn);
         if (!exitconn->marked_for_close)
@@ -455,7 +458,7 @@ void dns_cancel_pending_resolve(char *address) {
       connection_edge_end(pendconn, END_STREAM_REASON_RESOURCELIMIT,
                           pendconn->cpath_layer);
     }
-    circ = circuit_get_by_conn(pendconn);
+    circ = circuit_get_by_edge_conn(pendconn);
     if (circ)
       circuit_detach_stream(circ, pendconn);
     connection_free(pendconn);
@@ -553,11 +556,11 @@ static void dns_found_answer(char *address, uint32_t addr, char outcome) {
       if (pendconn->purpose == EXIT_PURPOSE_CONNECT) {
         connection_edge_end(pendconn, END_STREAM_REASON_RESOLVEFAILED, pendconn->cpath_layer);
         /* This detach must happen after we send the end cell. */
-        circuit_detach_stream(circuit_get_by_conn(pendconn), pendconn);
+        circuit_detach_stream(circuit_get_by_edge_conn(pendconn), pendconn);
       } else {
         send_resolved_cell(pendconn, RESOLVED_TYPE_ERROR);
         /* This detach must happen after we send the resolved cell. */
-        circuit_detach_stream(circuit_get_by_conn(pendconn), pendconn);
+        circuit_detach_stream(circuit_get_by_edge_conn(pendconn), pendconn);
       }
       connection_free(pendconn);
     } else {
@@ -565,12 +568,13 @@ static void dns_found_answer(char *address, uint32_t addr, char outcome) {
         /* prevent double-remove. */
         pend->conn->state = EXIT_CONN_STATE_CONNECTING;
 
-        circ = circuit_get_by_conn(pend->conn);
+        circ = circuit_get_by_edge_conn(pend->conn);
         tor_assert(circ);
         /* unlink pend->conn from resolving_streams, */
         circuit_detach_stream(circ, pend->conn);
         /* and link it to n_streams */
         pend->conn->next_stream = circ->n_streams;
+        pend->conn->on_circuit = circ;
         circ->n_streams = pend->conn;
 
         connection_exit_connect(pend->conn);
@@ -579,7 +583,7 @@ static void dns_found_answer(char *address, uint32_t addr, char outcome) {
          * but it does the right thing. */
         pendconn->state = EXIT_CONN_STATE_RESOLVEFAILED;
         send_resolved_cell(pendconn, RESOLVED_TYPE_IPV4);
-        circ = circuit_get_by_conn(pendconn);
+        circ = circuit_get_by_edge_conn(pendconn);
         tor_assert(circ);
         circuit_detach_stream(circ, pendconn);
         connection_free(pendconn);
@@ -633,7 +637,7 @@ int connection_dns_process_inbuf(connection_t *conn) {
   tor_assert(conn->type == CONN_TYPE_DNSWORKER);
 
   if (conn->state != DNSWORKER_STATE_BUSY && buf_datalen(conn->inbuf)) {
-    log_fn(LOG_WARN,"Bug: read data from an idle dns worker.  Please report.");
+    log_fn(LOG_WARN,"Bug: read data (%d bytes) from an idle dns worker.  Please report.", (int)buf_datalen(conn->inbuf));
 #ifdef TOR_FRAGILE
     tor_assert(0);
 #endif
@@ -728,11 +732,13 @@ static int dnsworker_main(void *data) {
         log_fn(LOG_INFO,"DNS worker exiting because of error on connection to Tor process.");
         log_fn(LOG_INFO,"(Error on %d was %s)", fd, tor_socket_strerror(tor_socket_errno(fd)));
       }
+      tor_close_socket(fd);
       spawn_exit();
     }
 
     if (address_len && read_all(fd, address, address_len, 1) != address_len) {
       log_fn(LOG_ERR,"read hostname failed. Child exiting.");
+      tor_close_socket(fd);
       spawn_exit();
     }
     address[address_len] = 0; /* null terminate it */
@@ -759,6 +765,7 @@ static int dnsworker_main(void *data) {
     set_uint32(answer+1, ip);
     if (write_all(fd, answer, 5, 1) != 5) {
       log_fn(LOG_ERR,"writing answer failed. Child exiting.");
+      tor_close_socket(fd);
       spawn_exit();
     }
   }
