@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: main.c,v 1.480 2005/04/01 02:37:40 nickm Exp $ */
-const char main_c_id[] = "$Id: main.c,v 1.480 2005/04/01 02:37:40 nickm Exp $";
+/* $Id: main.c,v 1.486 2005/04/08 06:16:09 arma Exp $ */
+const char main_c_id[] = "$Id: main.c,v 1.486 2005/04/08 06:16:09 arma Exp $";
 
 /**
  * \file main.c
@@ -142,7 +142,7 @@ int connection_add(connection_t *conn) {
   nfds++;
 
   log_fn(LOG_INFO,"new conn type %s, socket %d, nfds %d.",
-      CONN_TYPE_TO_STRING(conn->type), conn->s, nfds);
+      conn_type_to_string(conn->type), conn->s, nfds);
 
   return 0;
 }
@@ -158,7 +158,7 @@ int connection_remove(connection_t *conn) {
   tor_assert(nfds>0);
 
   log_fn(LOG_INFO,"removing socket %d (type %s), nfds now %d",
-         conn->s, CONN_TYPE_TO_STRING(conn->type), nfds-1);
+         conn->s, conn_type_to_string(conn->type), nfds-1);
 
   tor_assert(conn->poll_index >= 0);
   current_index = conn->poll_index;
@@ -322,14 +322,11 @@ void connection_start_writing(connection_t *conn) {
            (int)conn->s);
 }
 
-/** Close all connections that have been schedule to get closed */
+/** Close all connections that have been scheduled to get closed */
 static void
 close_closeable_connections(void)
 {
   int i;
-  if (!smartlist_len(closeable_connection_lst))
-    return;
-
   for (i = 0; i < smartlist_len(closeable_connection_lst); ) {
     connection_t *conn = smartlist_get(closeable_connection_lst, i);
     if (conn->poll_index < 0) {
@@ -353,13 +350,12 @@ conn_read_callback(int fd, short event, void *_conn)
   log_fn(LOG_DEBUG,"socket %d wants to read.",conn->s);
 
   assert_connection_ok(conn, time(NULL));
-  assert_all_pending_dns_resolves_ok();
 
   if (connection_handle_read(conn) < 0) {
     if (!conn->marked_for_close) {
 #ifndef MS_WINDOWS
       log_fn(LOG_WARN,"Bug: unhandled error on read for %s connection (fd %d); removing",
-             CONN_TYPE_TO_STRING(conn->type), conn->s);
+             conn_type_to_string(conn->type), conn->s);
 #ifdef TOR_FRAGILE
       tor_assert(0);
 #endif
@@ -370,7 +366,6 @@ conn_read_callback(int fd, short event, void *_conn)
     }
   }
   assert_connection_ok(conn, time(NULL));
-  assert_all_pending_dns_resolves_ok();
 
   if (smartlist_len(closeable_connection_lst))
     close_closeable_connections();
@@ -387,13 +382,12 @@ static void conn_write_callback(int fd, short events, void *_conn)
     return;
 
   assert_connection_ok(conn, time(NULL));
-  assert_all_pending_dns_resolves_ok();
 
   if (connection_handle_write(conn) < 0) {
     if (!conn->marked_for_close) {
       /* this connection is broken. remove it. */
       log_fn(LOG_WARN,"Bug: unhandled error on write for %s connection (fd %d); removing",
-             CONN_TYPE_TO_STRING(conn->type), conn->s);
+             conn_type_to_string(conn->type), conn->s);
 #ifdef TOR_FRAGILE
       tor_assert(0);
 #endif
@@ -403,7 +397,6 @@ static void conn_write_callback(int fd, short events, void *_conn)
     }
   }
   assert_connection_ok(conn, time(NULL));
-  assert_all_pending_dns_resolves_ok();
 
   if (smartlist_len(closeable_connection_lst))
     close_closeable_connections();
@@ -435,7 +428,7 @@ static int conn_close_if_marked(int i) {
       log_fn(LOG_INFO,
         "Conn (addr %s, fd %d, type %s, state %d) marked, but wants to flush %d bytes. "
         "(Marked at %s:%d)",
-        conn->address, conn->s, CONN_TYPE_TO_STRING(conn->type), conn->state,
+        conn->address, conn->s, conn_type_to_string(conn->type), conn->state,
         (int)conn->outbuf_flushlen, conn->marked_for_close_file, conn->marked_for_close);
     if (connection_speaks_cells(conn)) {
       if (conn->state == OR_CONN_STATE_OPEN) {
@@ -453,7 +446,7 @@ static int conn_close_if_marked(int i) {
     }
     if (connection_wants_to_flush(conn)) {
       log_fn(LOG_NOTICE,"Conn (addr %s, fd %d, type %s, state %d) is being closed, but there are still %d bytes we can't write. (Marked at %s:%d)",
-             conn->address, conn->s, CONN_TYPE_TO_STRING(conn->type), conn->state,
+             conn->address, conn->s, conn_type_to_string(conn->type), conn->state,
              (int)buf_datalen(conn->outbuf), conn->marked_for_close_file,
              conn->marked_for_close);
     }
@@ -478,7 +471,7 @@ void directory_all_unreachable(time_t now) {
                                               AP_CONN_STATE_CIRCUIT_WAIT))) {
     log_fn(LOG_NOTICE,"Network down? Failing connection to '%s:%d'.",
            conn->socks_request->address, conn->socks_request->port);
-    connection_close_unattached_ap(conn, END_STREAM_REASON_NET_UNREACHABLE);
+    connection_mark_unattached_ap(conn, END_STREAM_REASON_NET_UNREACHABLE);
   }
 }
 
@@ -832,23 +825,10 @@ static void second_elapsed_callback(int fd, short event, void *args)
   else
     circuit_note_clock_jumped(seconds_elapsed);
 
-  assert_all_pending_dns_resolves_ok();
   run_scheduled_events(now.tv_sec);
-  assert_all_pending_dns_resolves_ok();
 
   current_second = now.tv_sec; /* remember which second it is, for next time */
 
-#if 0
-  for (i=0;i<nfds;i++) {
-    conn = connection_array[i];
-    if (connection_has_pending_tls_data(conn) &&
-        connection_is_reading(conn)) {
-      log_fn(LOG_DEBUG,"sock %d has pending bytes.",conn->s);
-      return; /* has pending bytes to read; don't let poll wait. */
-    }
-  }
-#endif
-
   if (evtimer_add(timeout_event, &one_second))
     log_fn(LOG_ERR,
            "Error from libevent when setting one-second timeout event");
@@ -1071,8 +1051,8 @@ dumpstats(int severity) {
   for (i=0;i<nfds;i++) {
     conn = connection_array[i];
     log(severity, "Conn %d (socket %d) type %d (%s), state %d (%s), created %d secs ago",
-      i, conn->s, conn->type, CONN_TYPE_TO_STRING(conn->type),
-      conn->state, conn_state_to_string[conn->type][conn->state], (int)(now - conn->timestamp_created));
+      i, conn->s, conn->type, conn_type_to_string(conn->type),
+        conn->state, conn_state_to_string(conn->type, conn->state), (int)(now - conn->timestamp_created));
     if (!connection_is_listener(conn)) {
       log(severity,"Conn %d is to '%s:%d'.",i,conn->address, conn->port);
       log(severity,"Conn %d: %d bytes waiting on inbuf (last read %d secs ago)",i,
