@@ -1,8 +1,8 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: or.h,v 1.579 2005/03/31 19:26:33 arma Exp $ */
+/* $Id: or.h,v 1.593 2005/04/08 03:36:39 nickm Exp $ */
 
 /**
  * \file or.h
@@ -12,7 +12,7 @@
 
 #ifndef __OR_H
 #define __OR_H
-#define OR_H_ID "$Id: or.h,v 1.579 2005/03/31 19:26:33 arma Exp $"
+#define OR_H_ID "$Id: or.h,v 1.593 2005/04/08 03:36:39 nickm Exp $"
 
 #include "orconfig.h"
 #ifdef MS_WINDOWS
@@ -154,7 +154,9 @@
 #define MAX_NICKNAME_LEN 19
 /* Hex digest plus dollar sign. */
 #define MAX_HEX_NICKNAME_LEN (HEX_DIGEST_LEN+1)
-#define MAX_DIR_SIZE 500000
+/** Maximum size, in bytes, for resized buffers. */
+#define MAX_BUF_SIZE ((1<<24)-1)
+#define MAX_DIR_SIZE MAX_BUF_SIZE
 
 /* For http parsing */
 #define MAX_HEADERS_SIZE 50000
@@ -629,6 +631,7 @@ struct connection_t {
   int done_receiving; /**< For half-open connections; not used currently. */
   char has_sent_end; /**< For debugging: set once we've set the stream end,
                         and check in circuit_about_to_close_connection(). */
+  struct circuit_t *on_circuit; /**< DOCDOC */
 
   /* Used only by AP connections */
   socks_request_t *socks_request; /**< SOCKS structure describing request (AP
@@ -1164,10 +1167,15 @@ void onion_append_to_cpath(crypt_path_t **head_ptr, crypt_path_t *new_hop);
 
 /********************************* circuitlist.c ***********************/
 
-extern const char *circuit_state_to_string[];
+const char *circuit_state_to_string(int state);
+enum which_conn_changed_t { P_CONN_CHANGED=1, N_CONN_CHANGED=0 };
+void circuit_set_circid_orconn(circuit_t *circ, uint16_t id,
+                               connection_t *conn,
+                               enum which_conn_changed_t which);
 void circuit_close_all_marked(void);
 circuit_t *circuit_new(uint16_t p_circ_id, connection_t *p_conn);
-circuit_t *circuit_get_by_circ_id_conn(uint16_t circ_id, connection_t *conn);
+circuit_t *circuit_get_by_circid_orconn(uint16_t circ_id, connection_t *conn);
+circuit_t *circuit_get_by_edge_conn(connection_t *conn);
 circuit_t *circuit_get_by_conn(connection_t *conn);
 circuit_t *circuit_get_by_global_id(uint32_t id);
 circuit_t *circuit_get_by_rend_query_and_purpose(const char *rend_query, uint8_t purpose);
@@ -1177,19 +1185,10 @@ circuit_t *circuit_get_rendezvous(const char *cookie);
 circuit_t *circuit_get_clean_open(uint8_t purpose, int need_uptime,
                                   int need_capacity, int internal);
 void circuit_mark_all_unused_circs(void);
-int _circuit_mark_for_close(circuit_t *circ);
+void _circuit_mark_for_close(circuit_t *circ, int line, const char *file);
 
-#define circuit_mark_for_close(c)                                       \
-  do {                                                                  \
-    if (_circuit_mark_for_close(c)<0) {                                 \
-      log(LOG_WARN,"Duplicate call to circuit_mark_for_close at %s:%d (first at %s:%d)", \
-          _SHORT_FILE_,__LINE__,                                        \
-          c->marked_for_close_file,c->marked_for_close);                \
-    } else {                                                            \
-      c->marked_for_close_file = _SHORT_FILE_;                          \
-      c->marked_for_close = __LINE__;                                   \
-    }                                                                   \
-  } while (0)
+#define circuit_mark_for_close(c) \
+  _circuit_mark_for_close((c), __LINE__, _SHORT_FILE_)
 
 void assert_cpath_layer_ok(const crypt_path_t *cp);
 void assert_circuit_ok(const circuit_t *c);
@@ -1255,11 +1254,8 @@ int save_current_config(void);
 
 /********************************* connection.c ***************************/
 
-#define CONN_TYPE_TO_STRING(t) (((t) < _CONN_TYPE_MIN || (t) > _CONN_TYPE_MAX) ? \
-  "Unknown" : conn_type_to_string[(t)])
-
-extern const char *conn_type_to_string[];
-extern const char *conn_state_to_string[][_CONN_TYPE_MAX+1];
+const char *conn_type_to_string(int type);
+const char *conn_state_to_string(int type, int state);
 
 connection_t *connection_new(int type);
 void connection_unregister(connection_t *conn);
@@ -1267,19 +1263,10 @@ void connection_free(connection_t *conn);
 void connection_free_all(void);
 void connection_about_to_close_connection(connection_t *conn);
 void connection_close_immediate(connection_t *conn);
-int _connection_mark_for_close(connection_t *conn);
+void _connection_mark_for_close(connection_t *conn,int line, const char *file);
 
-#define connection_mark_for_close(c)                                    \
-  do {                                                                  \
-    if (_connection_mark_for_close(c)<0) {                              \
-      log(LOG_WARN,"Duplicate call to connection_mark_for_close at %s:%d (first at %s:%d)", \
-          _SHORT_FILE_,__LINE__,                                        \
-          c->marked_for_close_file,c->marked_for_close);                \
-    } else {                                                            \
-      c->marked_for_close_file = _SHORT_FILE_;                          \
-      c->marked_for_close = __LINE__;                                   \
-    }                                                                   \
-  } while (0)
+#define connection_mark_for_close(c) \
+  _connection_mark_for_close((c), __LINE__, _SHORT_FILE_)
 
 void connection_expire_held_open(void);
 
@@ -1296,9 +1283,10 @@ int connection_fetch_from_buf(char *string, size_t len, connection_t *conn);
 int connection_wants_to_flush(connection_t *conn);
 int connection_outbuf_too_full(connection_t *conn);
 int connection_handle_write(connection_t *conn);
+void _connection_controller_force_write(connection_t *conn);
 void connection_write_to_buf(const char *string, size_t len, connection_t *conn);
 
-connection_t *connection_exact_get_by_addr_port(uint32_t addr, uint16_t port);
+connection_t *connection_or_exact_get_by_addr_port(uint32_t addr, uint16_t port);
 connection_t *connection_get_by_identity_digest(const char *digest, int type);
 connection_t *connection_get_by_global_id(uint32_t id);
 
@@ -1324,7 +1312,11 @@ int connection_or_nonopen_was_started_here(connection_t *conn);
 
 /********************************* connection_edge.c ***************************/
 
-void connection_close_unattached_ap(connection_t *conn, int endreason);
+#define connection_mark_unattached_ap(conn, endreason) \
+  _connection_mark_unattached_ap((conn), (endreason), __LINE__, _SHORT_FILE_)
+
+void _connection_mark_unattached_ap(connection_t *conn, int endreason,
+                                    int line, const char *file);
 int connection_edge_reached_eof(connection_t *conn);
 int connection_edge_process_inbuf(connection_t *conn, int package_partial);
 int connection_edge_destroy(uint16_t circ_id, connection_t *conn);
@@ -1421,6 +1413,7 @@ typedef enum or_conn_status_event_t {
   OR_CONN_EVENT_CLOSED       = 3,
 } or_conn_status_event_t;
 
+void adjust_event_log_severity(void);
 int connection_control_finished_flushing(connection_t *conn);
 int connection_control_reached_eof(connection_t *conn);
 int connection_control_process_inbuf(connection_t *conn);
@@ -1487,6 +1480,10 @@ size_t dirserv_get_directory(const char **cp, int compress);
 size_t dirserv_get_runningrouters(const char **rr, int compress);
 void dirserv_set_cached_directory(const char *directory, time_t when,
                                   int is_running_routers);
+void dirserv_orconn_tls_done(const char *address,
+                             uint16_t or_port,
+                             const char *digest_rcvd,
+                             const char *nickname);
 void dirserv_free_all(void);
 
 /********************************* dns.c ***************************/
@@ -1796,7 +1793,7 @@ void free_trusted_dir_servers(void);
 routerinfo_t *routerinfo_copy(const routerinfo_t *router);
 void router_mark_as_down(const char *digest);
 void routerlist_remove_old_routers(int age);
-int router_load_single_router(const char *s);
+int router_load_single_router(const char *s, const char **msg);
 int router_load_routerlist_from_directory(const char *s,crypto_pk_env_t *pkey,
                                         int dir_is_recent, int dir_is_cached);
 addr_policy_result_t router_compare_addr_to_addr_policy(uint32_t addr,
