@@ -1,7 +1,7 @@
-/* Copyright 2004 Roger Dingledine, Nick Mathewson. */
+/* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendclient.c,v 1.81 2005/03/27 06:37:56 arma Exp $ */
-const char rendclient_c_id[] = "$Id: rendclient.c,v 1.81 2005/03/27 06:37:56 arma Exp $";
+/* $Id: rendclient.c,v 1.83 2005/04/02 22:11:24 nickm Exp $ */
+const char rendclient_c_id[] = "$Id: rendclient.c,v 1.83 2005/04/02 22:11:24 nickm Exp $";
 
 /**
  * \file rendclient.c
@@ -394,12 +394,12 @@ void rend_client_desc_here(char *query) {
       if (connection_ap_handshake_attach_circuit(conn) < 0) {
         /* it will never work */
         log_fn(LOG_WARN,"attaching to a rend circ failed. Closing conn.");
-        connection_close_unattached_ap(conn, END_STREAM_REASON_CANT_ATTACH);
+        connection_mark_unattached_ap(conn, END_STREAM_REASON_CANT_ATTACH);
       }
       tor_assert(conn->state != AP_CONN_STATE_RENDDESC_WAIT); /* avoid loop */
     } else { /* 404, or fetch didn't get that far */
       log_fn(LOG_NOTICE,"Closing stream for '%s.onion': hidden service is unavailable (try again later).", query);
-      connection_close_unattached_ap(conn, END_STREAM_REASON_TIMEOUT);
+      connection_mark_unattached_ap(conn, END_STREAM_REASON_TIMEOUT);
     }
   }
 }
