@@ -1,7 +1,7 @@
-/* Copyright 2004 Roger Dingledine, Nick Mathewson. */
+/* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendservice.c,v 1.120 2005/03/23 06:21:48 nickm Exp $ */
-const char rendservice_c_id[] = "$Id: rendservice.c,v 1.120 2005/03/23 06:21:48 nickm Exp $";
+/* $Id: rendservice.c,v 1.122 2005/04/07 21:07:19 nickm Exp $ */
+const char rendservice_c_id[] = "$Id: rendservice.c,v 1.122 2005/04/07 21:07:19 nickm Exp $";
 
 /**
  * \file rendservice.c
@@ -988,7 +988,7 @@ rend_service_dump_stats(int severity)
         continue;
       }
       log(severity, "  Intro point at %s: circuit is %s",nickname,
-          circuit_state_to_string[circ->state]);
+          circuit_state_to_string(circ->state));
     }
   }
 }
