@@ -1,9 +1,9 @@
 /* Copyright 2001 Matej Pfajfar.
  * Copyright 2001-2004 Roger Dingledine.
- * Copyright 2004 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: routerlist.c,v 1.222 2005/04/01 08:42:26 arma Exp $ */
-const char routerlist_c_id[] = "$Id: routerlist.c,v 1.222 2005/04/01 08:42:26 arma Exp $";
+/* $Id: routerlist.c,v 1.226 2005/04/06 06:17:35 arma Exp $ */
+const char routerlist_c_id[] = "$Id: routerlist.c,v 1.226 2005/04/06 06:17:35 arma Exp $";
 
 #include "or.h"
 
@@ -288,6 +288,10 @@ void routerlist_add_family(smartlist_t *sl, routerinfo_t *router) {
   }
 }
 
+/** List of string for nicknames we've warned about and haven't yet succeeded.
+ */
+static smartlist_t *warned_nicknames = NULL;
+
 /** Given a comma-and-whitespace separated list of nicknames, see which
  * nicknames in <b>list</b> name routers in our routerlist that are
  * currently running.  Add the routerinfos for those routers to <b>sl</b>.
@@ -303,25 +307,39 @@ add_nickname_list_to_smartlist(smartlist_t *sl, const char *list, int warn_if_do
   tor_assert(sl);
 
   nickname_list = smartlist_create();
+  if (!warned_nicknames)
+    warned_nicknames = smartlist_create();
 
   smartlist_split_string(nickname_list, list, ",",
                          SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK, 0);
 
   SMARTLIST_FOREACH(nickname_list, const char *, nick, {
+    int warned;
     if (!is_legal_nickname_or_hexdigest(nick)) {
       log_fn(LOG_WARN,"Nickname %s is misformed; skipping", nick);
       continue;
     }
     router = router_get_by_nickname(nick);
+    warned = smartlist_string_isin(warned_nicknames, nick);
     if (router) {
-      if (router->is_running)
+      if (router->is_running) {
         smartlist_add(sl,router);
-      else
-        log_fn(warn_if_down ? LOG_WARN : LOG_DEBUG,
-               "Nickname list includes '%s' which is known but down.",nick);
-    } else
-      log_fn(has_fetched_directory ? LOG_WARN : LOG_INFO,
-             "Nickname list includes '%s' which isn't a known router.",nick);
+        if (warned)
+          smartlist_string_remove(warned_nicknames, nick);
+      } else {
+        if (!warned) {
+          log_fn(warn_if_down ? LOG_WARN : LOG_DEBUG,
+                 "Nickname list includes '%s' which is known but down.",nick);
+          smartlist_add(warned_nicknames, tor_strdup(nick));
+        }
+      }
+    } else {
+      if (!warned) {
+        log_fn(has_fetched_directory ? LOG_WARN : LOG_INFO,
+               "Nickname list includes '%s' which isn't a known router.",nick);
+        smartlist_add(warned_nicknames, tor_strdup(nick));
+      }
+    }
   });
   SMARTLIST_FOREACH(nickname_list, char *, nick, tor_free(nick));
   smartlist_free(nickname_list);
@@ -484,20 +502,25 @@ routerinfo_t *router_choose_random_node(const char *preferred,
   excludednodes = smartlist_create();
   add_nickname_list_to_smartlist(excludednodes,excluded,0);
 
-  /* try the preferred nodes first */
+  /* Try the preferred nodes first. Ignore need_uptime and need_capacity,
+   * since the user explicitly asked for these nodes. */
   sl = smartlist_create();
   add_nickname_list_to_smartlist(sl,preferred,1);
   smartlist_subtract(sl,excludednodes);
   if (excludedsmartlist)
     smartlist_subtract(sl,excludedsmartlist);
+#if 0
   if (need_uptime)
     routerlist_sl_remove_unreliable_routers(sl);
   if (need_capacity)
     choice = routerlist_sl_choose_by_bandwidth(sl);
   else
-    choice = smartlist_choose(sl);
+#endif
+  choice = smartlist_choose(sl);
   smartlist_free(sl);
   if (!choice && !strict) {
+    /* Then give up on our preferred choices: any node
+     * will do that has the required attributes. */
     sl = smartlist_create();
     router_add_running_routers_to_smartlist(sl, allow_unverified,
                                             need_uptime, need_capacity);
@@ -733,6 +756,11 @@ void routerlist_free_current(void)
   if (routerlist)
     routerlist_free(routerlist);
   routerlist = NULL;
+  if (warned_nicknames) {
+    SMARTLIST_FOREACH(warned_nicknames, char *, cp, tor_free(cp));
+    smartlist_free(warned_nicknames);
+    warned_nicknames = NULL;
+  }
 }
 
 void free_trusted_dir_servers(void)
@@ -769,9 +797,11 @@ void router_mark_as_down(const char *digest) {
  * their pointers to <b>router</b> after invoking this function; <b>router</b>
  * will either be inserted into the routerlist or freed.  Returns 0 if the
  * router was added; -1 if it was not.
+ *
+ * DOCDOC msg
  */
 static int
-router_add_to_routerlist(routerinfo_t *router) {
+router_add_to_routerlist(routerinfo_t *router, const char **msg) {
   int i;
   routerinfo_t *r;
   char id_digest[DIGEST_LEN];
@@ -797,11 +827,12 @@ router_add_to_routerlist(routerinfo_t *router) {
         smartlist_set(routerlist->routers, i, router);
         return 0;
       } else {
-        log_fn(LOG_DEBUG, "Skipping old entry for router '%s'",
+        log_fn(LOG_DEBUG, "Skipping not-new descriptor for router '%s'",
                router->nickname);
         /* Update the is_running status to whatever we were told. */
         r->is_running = router->is_running;
         routerinfo_free(router);
+        if (msg) *msg = "Router descriptor was not new.";
         return -1;
       }
     } else if (!strcasecmp(router->nickname, r->nickname)) {
@@ -827,6 +858,7 @@ router_add_to_routerlist(routerinfo_t *router) {
         log_fn(LOG_DEBUG, "Skipping unverified entry for verified router '%s'",
                router->nickname);
         routerinfo_free(router);
+        if (msg) *msg = "Already have verified router with different key and same nickname";
         return -1;
       }
     }
@@ -865,15 +897,19 @@ routerlist_remove_old_routers(int age)
 }
 
 /*
- * Code to parse router descriptors and directories.
+ * Code to parse a single router descriptors and insert it into the
+ * directory.  Return -1 if the descriptor was ill-formed; 0 if the
+ * descriptor was well-formed but could not be added; and 1 if the
+ * descriptor was added.
  */
 int
-router_load_single_router(const char *s)
+router_load_single_router(const char *s, const char **msg)
 {
   routerinfo_t *ri;
 
   if (!(ri = router_parse_entry_from_string(s, NULL))) {
     log_fn(LOG_WARN, "Error parsing router descriptor; dropping.");
+    if (msg) *msg = "Couldn't parse router descriptor";
     return -1;
   }
   if (routerlist && routerlist->running_routers) {
@@ -883,9 +919,10 @@ router_load_single_router(const char *s)
                                         rr->running_routers,
                                         rr->is_running_routers_format);
   }
-  if (router_add_to_routerlist(ri)<0) {
+  if (router_add_to_routerlist(ri, msg)<0) {
     log_fn(LOG_WARN, "Couldn't add router to list; dropping.");
-    return -1;
+    if (msg && !*msg) *msg = "Couldn't add router to list.";
+    return 0;
   } else {
     smartlist_t *changed = smartlist_create();
     smartlist_add(changed, ri);
@@ -893,7 +930,7 @@ router_load_single_router(const char *s)
     smartlist_free(changed);
   }
   log_fn(LOG_DEBUG, "Added router to list");
-  return 0;
+  return 1;
 }
 
 /** Add to the current routerlist each router stored in the
@@ -922,7 +959,7 @@ int router_load_routerlist_from_directory(const char *s,
     smartlist_t *changed = smartlist_create();
     SMARTLIST_FOREACH(new_list->routers, routerinfo_t *, r,
     {
-      if (router_add_to_routerlist(r)==0)
+      if (router_add_to_routerlist(r,NULL)==0)
         smartlist_add(changed, r);
     });
     smartlist_clear(new_list->routers);
